package jarnik.ld20
{
	import org.flixel.FlxState;
	import org.flixel.FlxSprite;
	import org.flixel.FlxText;
	import org.flixel.FlxPoint;
	import org.flixel.FlxG;

	public class CutsceneState extends FlxState
	{
        private var speech:FlxText;
        private var image:FlxSprite;
        private var endImage:Boolean = false;

		override public function create():void
		{
            /// show level end image, if available
            if ( Levels.currentLevel>0 )
                if ( Levels.levels[Levels.currentLevel-1].endImage && Levels.endImagesShown.indexOf( Levels.currentLevel-1 ) == -1 ) {
                    endImage = true;

                    image = new FlxSprite( 0,0, Levels.levels[Levels.currentLevel-1].endImage );
                    add( image );
                    image.x = (FlxG.width-image.width)/2;
                    image.y = (FlxG.height-image.height)/2;

                    var endText:FlxText = new FlxText(10,10,FlxG.width-20, Levels.levels[Levels.currentLevel-1].endText );
                    add( endText );

                    add( new FlxText(5,FlxG.height-17,200,"PRESS X TO CONTINUE" ));
                }


            if ( !endImage && (Levels.currentLevel == -1 || Levels.currentLevel>=Levels.levels.length)){
                // add dialogue screen
                var dialogue:FlxSprite;
                add( dialogue = new FlxSprite(0,0,Assets.ImgDialogue) );
                dialogue.scale = new FlxPoint(3,3);
                dialogue.x = 107;
                dialogue.y = 80;

                var firstLine:String = ((Levels.currentLevel == -1) ? Levels.introDialogue[0] : Levels.outroDialogue[0]);
                speech = new FlxText(150,10,FlxG.width-150-20, firstLine);
                add( speech );
                
                add( new FlxText(5,FlxG.height-20,200,"PRESS X TO CONTINUE" ));

                FlxG.flash(0xff000000, 0.5 );
            }
		}

        private function endCutscene():void {
            if ( endImage ) {
                Levels.endImagesShown.push( Levels.currentLevel-1 );
                FlxG.switchState( new CutsceneState() );
                return;
            }

            Levels.currentDialogueLine = 0;
            if ( Levels.currentLevel == -1 ) {
                Levels.currentLevel=0;
            } else {
                Levels.currentLevel = -1;
                Levels.endImagesShown = [];
                FlxG.switchState( new TitleState() );        
            }
        }

        private function nextLevel():void {
            var levelInfo:Object = Levels.levels[Levels.currentLevel];
			FlxG.switchState(new PlayState(
                levelInfo.name,
                levelInfo.subtitle,
                levelInfo.enemy,
                levelInfo.terrain,
                levelInfo.length
            ));		
        }

		override public function update():void
		{
			super.update();

            if ( endImage ) {
                if ( FlxG.keys.justReleased("X") )
                    FlxG.fade(0xff000000, 0.5, endCutscene );
                return;
            }

            if (Levels.currentLevel == -1 || Levels.currentLevel>=Levels.levels.length) {
                if ( FlxG.keys.justReleased("X") ) {
                    var dialogArray:Array = ( Levels.currentLevel == -1 ? Levels.introDialogue : Levels.outroDialogue );

                    Levels.currentDialogueLine++;
                    if ( Levels.currentDialogueLine < dialogArray.length ) {
                        FlxG.play( Assets.SndCharged );
                        if ( dialogArray[Levels.currentDialogueLine] is Class ) {
                            speech.text = "";
                            if ( image )
                                remove( image );
                            image = new FlxSprite( 0,0, dialogArray[Levels.currentDialogueLine] );
                            add( image );
                            image.x = (FlxG.width-image.width)/2;
                            image.y = (FlxG.height-image.height)/2;
                        } else { 
                            if ( image )
                                image.visible = false;
                            speech.text = dialogArray[Levels.currentDialogueLine];
                        }
                    } else
                        FlxG.fade(0xff000000, 0.5, endCutscene );
                }
            } else
                nextLevel();
		}
	}
}
