package jarnik.ld20
{
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxState;
	import org.flixel.FlxObject;
	import org.flixel.FlxText;
	import org.flixel.FlxSprite;
	import org.flixel.FlxPoint;
	import org.flixel.FlxRect;
    import org.flixel.FlxEmitter;

	public class PlayState extends FlxState
	{
        private var heroes:FlxGroup;
        private var enemies:FlxGroup;
        private var blocks:FlxGroup;
        private var creatures:FlxGroup;
        private var clones:FlxGroup;
        private var emitter:FlxEmitter;
        private var levelFinished:Boolean = false;
        private var followedHero:Hero;
        private var counter:FlxText;
        private var levelTitle:FlxText;
        private var levelSubtitle:FlxText;
        public static var LEVEL_SIZE:int = 0;
        private var hideTimer:Number = 3;
        private var cooldownTimer:Number = 0;
        private var cloningTimer:Number = 0;
        private var magicItem:FlxSprite;
        private var cooldownProgress:FlxSprite;
        private var heroIcons:Array;
        private var cloning:Boolean = false;

        private const MAX_HEROES:int = 32;
        private const COOLDOWN_SPEED:Number = 3;
        private const CLONING_SPEED:Number = 2;

        private var _name:String;
        private var _subtitle:String;
        private var _enemies:Array;
        private var _terrain:Class;
        private var _length:Number;

        public function PlayState(
           Name: String, 
           Subtitle: String,
           Enemies: Array,
           Terrain: Class,
           Length: Number
        ):void {
           _name = Name; 
           _subtitle = Subtitle;
           _enemies = Enemies;
           _terrain = Terrain;
           _length = Length;
        }

		override public function create():void
		{
            FlxG.bgColor = 0xff101010;
            
            var info:FlxText = new FlxText(0,0,FlxG.width, "LEFT/RIGHT to move, SPACE to jump, Z to attack, press X and walk to clone yourself. Magic item needs a while to recharge after every cloning\n\nNOW GO RIGHT!");
            info.setFormat(null,8,0x404040);
            info.scrollFactor = new FlxPoint();
            if ( Levels.currentLevel==0 )
                add( info );

            add( blocks = new FlxGroup() );
            buildLevel( _terrain, _length );

            creatures = new FlxGroup();

            heroes = new FlxGroup();
            heroes.add( new Hero() );
            creatures.add( heroes );

            enemies = new FlxGroup();
            creatures.add( enemies );

            spawnEnemies( _enemies, _length );

            clones = new FlxGroup();
            creatures.add( clones );

            add( creatures );

            emitter = new FlxEmitter(100,100,10);
            emitter.makeParticles(Assets.ImgMeat,50,16,true);
            emitter.setXSpeed(-50,50);
            emitter.setYSpeed(-50,10);
            emitter.gravity = 150;
            add( emitter );
            creatures.add( emitter );

            add( counter = new FlxText(0,FlxG.height - 55, 150, "") );
            counter.scrollFactor = new FlxPoint();
            counter.visible = false;
            updateFollow();

            createHUD();

            add( levelTitle = new FlxText(0,50,FlxG.width,_name) );
            levelTitle.setFormat(null, 16, 0xffffff, "center", 0xff000000);
            levelTitle.scrollFactor = new FlxPoint();
            add( levelSubtitle = new FlxText(0,70,FlxG.width,_subtitle) );
            levelSubtitle.setFormat(null, 8, 0xd0d0d0, "center", 0xff000000);
            levelSubtitle.scrollFactor = new FlxPoint();

            FlxG.flash(0xff000000, 0.5 );
            FlxG.play( Assets.SndLevelStart );
            
		}

        private function createHUD():void {

            var backPane:FlxSprite = new FlxSprite(0,FlxG.height - 12);
            backPane.makeGraphic( FlxG.width, 12, 0xaa000000 );
            add( backPane );
            backPane.scrollFactor = new FlxPoint();

            add( magicItem = new FlxSprite(5, FlxG.height - 20, Assets.ImgMagicItem ) );
            magicItem.scrollFactor = new FlxPoint();
            var cooldownBack:FlxSprite = new FlxSprite( 25, FlxG.height - 10 );
            cooldownBack.makeGraphic( 50, 8, 0xaf000000, true );
            add( cooldownBack );
            cooldownBack.scrollFactor = new FlxPoint();

            var MAX_WARRIOR_STEPS:int = Math.round(Math.log(MAX_HEROES)*Math.LOG2E)+1;
            var icon:FlxSprite;
            heroIcons = [];
            for (var i:int = 0; i < MAX_WARRIOR_STEPS; i++ ) {
                add( icon = new FlxSprite( FlxG.width - i*10 - 10, FlxG.height - 10, Assets.ImgHumanIcon ));
                icon.scrollFactor = new FlxPoint();
                heroIcons.push( icon );
            }
        }

        private function updateHUD():void {
            // update icon count visibility
            var heroCount:int = Math.max(0,heroes.countLiving());
            heroCount += Math.max(0,clones.countLiving());
            var iconsVisible:int = Math.round(Math.log( heroCount )*Math.LOG2E)+1;
            for (var i:int = 0; i < heroIcons.length; i++ ) {
                (heroIcons[i] as FlxSprite).color = (i < iconsVisible ? 0xffffff: 0x101010 );
            }

            // update cooldown progress
            if ( !cooldownProgress ) {
                cooldownProgress = new FlxSprite( 25, FlxG.height - 10 );
                add( cooldownProgress );
                cooldownProgress.scrollFactor = new FlxPoint();
            }
            var cooldownSize:Number = Math.max(1,50*(1 - cooldownTimer / COOLDOWN_SPEED));
            if ( cloning )
                cooldownSize = Math.max(1,50*(cloningTimer / CLONING_SPEED));
            cooldownProgress.makeGraphic( cooldownSize, 8, (cooldownSize >= 50 && heroCount < MAX_HEROES ) ? 0xffff05d3 : 0xff808080 );

            // update magic item alpha
            var prevColor:int = magicItem.color;
            magicItem.color = ( (cooldownSize < 50) || (heroCount >= MAX_HEROES ) ? 0x303030 : 0xffffff );
            if ( prevColor != magicItem.color && magicItem.color == 0xffffff )
                FlxG.play( Assets.SndCharged );
        }

        private function spawnEnemies( Enemies:Array, levelLength:Number ):void {
            var spawnCount:int = Math.max(3,Math.round(levelLength / 7));
            for ( var i:int = 0; i < spawnCount; i++ )
                spawnEnemy( Enemies[Math.floor(Math.random()*Enemies.length)] );
        }

        private function spawnEnemy( Img:Class ):void {
            var spawnBounds:Number = 10;
            var X:Number = (Math.random()*(LEVEL_SIZE-spawnBounds*2) + spawnBounds )*16;            
            enemies.add( new Enemy( Img, X, 50, (Math.random()>0.5?FlxObject.LEFT:FlxObject.RIGHT) ) );
        }

        private function buildLevel(asset:Class, length:int = 10 ):void {
            LEVEL_SIZE = length;

            var level:int = 5;
            var step:int = 0;
            var block:FlxSprite;
            var border:int = 2;
            var groundLevel:int = FlxG.height;
            var maxStep:int = 1;
            for (var i:int = -border; i < (length+1); i++) {
                for (var j:int = 0; j < level; j++ ) {
                    block = new FlxSprite( 16 * i, groundLevel - 16 - j*16, asset )
                    block.solid = (j==level-1);
                    blocks.add( block );
                    block.immovable = true;
                }
                step = Math.floor(Math.random()*(1+2*maxStep))-maxStep;
                if ( i<0 || i >= (length-border) )
                    step = 0;
                level += step;
                level = Math.max( 1, Math.min( 10, level ) );
            }

            FlxG.worldBounds = new FlxRect( -border*16, 0, (LEVEL_SIZE + border)*16, FlxG.height + 20 );
        }

        override public function update():void {

            if ( levelFinished ) {
                super.update();
                FlxG.collide(blocks, creatures);
                return;
            }

            if ( hideTimer > 0 ) {
                hideTimer -= FlxG.elapsed;
                if ( hideTimer <= 0 )
                    hideTitle();
            }

            if ( cooldownTimer > 0 )
                cooldownTimer -= FlxG.elapsed;                
            if ( cloningTimer > 0 )
                cloningTimer -= FlxG.elapsed;

            var hero:Hero;
            var clone:Hero;

            // CLONING start
            if ( FlxG.keys.justPressed( "X" ) && cooldownTimer <=0 && heroes.countLiving()<MAX_HEROES ) {             
                cloning = true;
                cloningTimer = CLONING_SPEED;
                FlxG.play( Assets.SndClone );

                //pause world
                heroes.active = false;
                enemies.active = false;

                tintWorld(0x303030);

                clones.clear();
                //clone all living heroes
                for each ( hero in heroes.members ) {
                    if ( !hero.alive )
                        continue;
                    clone = new Hero( hero.x, hero.y, hero.facing );
                    clones.add( clone );
                }

                FlxG.flash(0xffffffff, 0.2 );
            }

            // CLONING end
            //if ( FlxG.keys.justReleased( "X" ) && cloning ) {
            if ( cloningTimer <= 0 && cloning ) {
                cloning = false;
                heroes.active = true;
                enemies.active = true;
                cooldownTimer = COOLDOWN_SPEED;                 
                tintWorld(0xffffff);
                FlxG.play( Assets.SndNextDialog );

                // add clones to main hero group
                for each ( hero in clones.members ) {
                    if ( hero.alive )
                        heroes.add( hero );
                }
                clones.clear();
                followedHero = null
                updateFollow();
            }
            if ( FlxG.keys.justPressed("Z") ) {
                for each ( hero in heroes.members ) {
                    if ( hero.alive ) 
                        hero.attack();
                }
            }

            //controls
            var controlledGroup:FlxGroup;
            //controll just the new clones until the button is released
            //if ( FlxG.keys.X )
            if ( cloning )
                controlledGroup = clones;
            else 
                controlledGroup = heroes;

            for each ( hero in controlledGroup.members ) {
                if ( !hero.alive )
                    continue;

                hero.acceleration.x = 0;
                if ( FlxG.keys.LEFT || FlxG.keys.RIGHT ) {      
                    if ( (FlxG.keys.LEFT && hero.x > 0) || (FlxG.keys.RIGHT && hero.x < 16*(LEVEL_SIZE-1)) )
                        hero.acceleration.x = hero.maxVelocity.x*4*( FlxG.keys.LEFT ? -1 : 1 );
                }
                if ( FlxG.keys.justPressed( "SPACE" ) )
                    hero.jump();
            }

            updateHUD();

            super.update();

            FlxG.collide(blocks, creatures);

            if ( heroes.active ) {
                FlxG.overlap( heroes, enemies, clash );
            }

            // check level finished            
            for each ( hero in controlledGroup.members ) {
                if ( !hero.alive )
                    continue;

                if ( hero.x < 16*(LEVEL_SIZE-1) || levelFinished )
                    continue;
                Levels.currentLevel++;
                FlxG.play( Assets.SndComplete );
                showMessage("LEVEL COMPLETE");
            }

            // check game over
            var livingCount:int = heroes.countLiving();
            counter.text = "WARRIORS: "+(livingCount + clones.members.length);
            if ( livingCount == 0 ) {
                showMessage("GAME OVER");
            }
           
        }

        private function tintWorld(c:int):void {
            heroes.setAll("color",c);
            enemies.setAll("color",c);
            blocks.setAll("color",c);
            emitter.setAll("color",c);
        }

        private function updateFollow():void {
            var willUpdate:Boolean = false;
            if ( followedHero ) {
                if ( !followedHero.alive )
                    willUpdate = true;
            } else 
                willUpdate = true;
            if ( !willUpdate )
                return;

            followedHero = getMedianHero();
            if ( followedHero ) {
                FlxG.camera.follow( followedHero );
                FlxG.camera.setBounds(0,0,LEVEL_SIZE*16,FlxG.height);
            }
        }

        private function getMedianHero():Hero {
            var median:int = Math.floor(heroes.countLiving()/2);
            if ( median<0 ) return null;
            var index:int = 0;
            for each ( var hero:Hero in heroes.members ) {
                if ( !hero.alive) continue;
                if (index==median)
                    return hero;
                index++;
            }
            return null;
        }

        private function showMessage(text:String):void {
            hideTitle();
            var message:FlxText;
            creatures.active = false;
            levelFinished = true;
            FlxG.fade(0xff000000, 2, onFinished, true);
            message = new FlxText(0,50,FlxG.width,text);
            message.setFormat(null,16,0xffffffff,"center",0xff000000);
            message.scrollFactor = new FlxPoint();
            add( message );
        }

        private function hideTitle():void {
            levelTitle.visible = false;
            levelSubtitle.visible = false;
        }

        private function clash( Object1:FlxObject, Object2:FlxObject ):void {
            if ( Object2.alive && Object1.alive && (Object1 is Hero) && (Object2 is Enemy) ) {
                if ( (Object1 as Hero).attacking  ) {
                    // kill
                    Object2.kill();
                    burst( Object2 );
                    FlxG.play( Assets.SndStab );
                } else {
                    /// or get killed
                    FlxG.play( Assets.SndDie );
                    Object1.kill();
                    burst( Object1 );
                    updateFollow();
                }
            }
        }

        private function burst( o:FlxObject ):void {
            emitter.x = o.x;
            emitter.y = o.y;
            emitter.start(true,5,0,10);
        }

        private function onFinished():void {
            FlxG.switchState( new CutsceneState() );
        }

	}
}
